(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: codomain.ml,v 1.18 2014-02-26 00:12:42 fclement Exp $ *)

module Nat = Unatural;;

let c_qual = Upervasives.qualify "Codomain";;

let c_failwith = Upervasives.failwith_qual c_qual;;

open Segmentation_spec;;

module Make =
  functor (T : Tuple_spec.TUPLE) ->
  functor (Cs : CODOMAIN_STRUCTURE) ->
struct
  include T;;
  include Cs;;
  
  (*let digest = Tuple.digest;;*)
  
  let all_channels = init Upervasives.id;;
  
  let list_rev_skip n l =
    let rec loop accu i = function
      | [] -> accu
      | x :: l ->
          if i = n then loop accu (Nat.succ i) l else
          loop (x :: accu) (Nat.succ i) l in
    loop [] Nat._0_ l
  ;;
  let complementary tup = init (fun i -> list_rev_skip i (to_list tup));;
  
  let of_color scalar_of_int c =
    map scalar_of_int (init (get_component_of_color c))
  ;;
  
  let to_color scalar_to_int t =
    let (r, g, b) = Tuple.Triple.map scalar_to_int (to_triple t) in
    Colors.color_of_rgb r g b
  ;;
end
;;

(* Single. *)

module Single_structure =
struct
  let get_component_of_color c i =
    match Nat.to_int i with
    | 0 ->
      let r = Colors.r_of c
      and g = Colors.g_of c
      and b = Colors.b_of c in
      if r = g && g = b then r else
      c_failwith "get_component_of_color"
        (Printf.sprintf "color %X is not grey" c)
    | _ -> assert false
  ;;
end
;;

module Single_Codomain = Make (Tuple.Single) (Single_structure);;

let single = (module Single_Codomain : CODOMAIN);;

(* Triple. *)

module Triple_structure =
struct
  let get_component_of_color c i =
    match Nat.to_int i with
    | 0 -> Colors.r_of c
    | 1 -> Colors.g_of c
    | 2 -> Colors.b_of c
    | _ -> assert false
  ;;
end
;;

module Triple_Codomain = Make (Tuple.Triple) (Triple_structure);;

let triple = (module Triple_Codomain : CODOMAIN);;

(* Vector. *)

module Vector_structure =
  functor (V : Vector_spec.VECTOR) ->
struct
  let get_component_of_color c n =
    if V.get_dim () = Nat._1_ then
      match Nat.to_int n with
      | 0 -> Colors.bri_of c
      | _ -> assert false else
    if V.get_dim () = Nat._3_ then
      match Nat.to_int n with
      | 0 -> Colors.r_of c
      | 1 -> Colors.g_of c
      | 2 -> Colors.b_of c
      | _ -> assert false else
    c_failwith "get_component_of_color" "wrong dimension"
  ;;
end
;;

module Vector_Codomain =
  functor (V : Vector_spec.VECTOR) ->
  Make (Tuple.Vector (V)) (Vector_structure (V))
;;

let vector_of_nat dim =
  let module Dim = struct let get_dim () = dim;; end in
  let module VecCod = Vector_Codomain (Vector.Make (Dim)) in
  (module VecCod : CODOMAIN)
;;
