(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 3.3+dev4 generated code *)

open Segmentation_types_print;;

let print_dimension ppf = function | u_t -> Unatural_print.print_t ppf u_t
;;

let print_cutting_choice ppf = function
  | Configuration_types.All_cuttings ->
    Format.fprintf ppf "%s" "All_cuttings"
  | Configuration_types.Best_cutting ->
    Format.fprintf ppf "%s" "Best_cutting"

and print_component_choice ppf = function
  | Configuration_types.All_components ->
    Format.fprintf ppf "%s" "All_components"
  | Configuration_types.Best_component ->
    Format.fprintf ppf "%s" "Best_component"
;;

let print_dimension_management ppf = function
  | (c, c0) ->
    Format.fprintf ppf "%,@[<1>(@,"; print_cutting_choice ppf c;
    Format.fprintf ppf "%,,@ "; print_component_choice ppf c0;
    Format.fprintf ppf "%,@,)@]"
;;

let print_stopping_criterion ppf = function
  | {
      Configuration_types.max_iteration = h_s;
      Configuration_types.min_best_scalar_indicator = f;
      Configuration_types.min_scalar_relative_error = f0;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>max_iteration =@ ";
    History_types_print.print_state_number ppf h_s;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>min_best_scalar_indicator =@ ";
    Lib_print.print_quoted_float ppf f; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>min_scalar_relative_error =@ ";
    Lib_print.print_quoted_float ppf f0; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@,}@]"
;;

let print_t ppf = function
  | {
      Configuration_types.dimension = d;
      Configuration_types.dimension_management = d0;
      Configuration_types.cutting_strategy = c;
      Configuration_types.stopping_criterion = s;
      Configuration_types.image_width = w;
      Configuration_types.image_height = h;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>dimension =@ "; print_dimension ppf d;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>dimension_management =@ ";
    print_dimension_management ppf d0; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>cutting_strategy =@ ";
    print_cutting_strategy ppf c; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>stopping_criterion =@ ";
    print_stopping_criterion ppf s; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>image_width =@ "; print_width ppf w;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>image_height =@ "; print_height ppf h;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let print_quantity_choice ppf = function
  | Configuration_types.Param_quantity ->
    Format.fprintf ppf "%s" "Param_quantity"
  | Configuration_types.All_quantities ->
    Format.fprintf ppf "%s" "All_quantities"

and print_channel_choice ppf = function
  | Configuration_types.Compound_channel ->
    Format.fprintf ppf "%s" "Compound_channel"
  | Configuration_types.All_channels ->
    Format.fprintf ppf "%s" "All_channels"
;;

let print_graphics_kind ppf = function
  | (q, c) ->
    Format.fprintf ppf "%,@[<1>(@,"; print_quantity_choice ppf q;
    Format.fprintf ppf "%,,@ "; print_channel_choice ppf c;
    Format.fprintf ppf "%,@,)@]"
;;

let print_graphics_mode ppf = function
  | { Configuration_types.on = b; Configuration_types.kind = g_o; } ->
    Format.fprintf ppf "%,@[<1>{@,"; Format.fprintf ppf "%,@[<1>on =@ ";
    Lib_print.print_bool ppf b; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>kind =@ ";
    Lib_print.print_Option print_graphics_kind ppf g_o;
    Format.fprintf ppf "%,;@]@ "; Format.fprintf ppf "%,@,}@]"
;;

let print_program_configuration ppf = function
  | {
      Configuration_types.formatter = f_f;
      Configuration_types.report_allocated_memory = b;
      Configuration_types.incremental_mode = b0;
      Configuration_types.graphics_mode = g;
      Configuration_types.graphics_window_min_size = s;
      Configuration_types.graphics_window_max_size = s0;
      Configuration_types.graphics_gradient_threshold = u_t;
      Configuration_types.final_image_file_name = u_t0;
    } ->
    Format.fprintf ppf "%,@[<1>{@,";
    Format.fprintf ppf "%,@[<1>formatter =@ ";
    Format_print.print_formatter ppf f_f; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>report_allocated_memory =@ ";
    Lib_print.print_bool ppf b; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>incremental_mode =@ ";
    Lib_print.print_bool ppf b0; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>graphics_mode =@ "; print_graphics_mode ppf g;
    Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>graphics_window_min_size =@ ";
    print_size ppf s; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>graphics_window_max_size =@ ";
    print_size ppf s0; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>graphics_gradient_threshold =@ ";
    Unatural_print.print_t ppf u_t; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@[<1>final_image_file_name =@ ";
    Ufilename_print.print_t ppf u_t0; Format.fprintf ppf "%,;@]@ ";
    Format.fprintf ppf "%,@,}@]"
;;

(* End of OCamlGen, 3.3+dev4 generated code *)
