(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: vector_spec.mli,v 1.23 2014-01-21 17:17:24 fclement Exp $ *)

open Vector_types_spec;;

type component_number = Unatural.t;;
  
module type VECTOR =
sig
  include DIMENSION;;
  
  type 'a t;;
  
  (*val digest : ('a, 'a t) Utype_digest.lifter;;*)
  
  val of_array : 'a array -> 'a t;;
  val to_array : 'a t -> 'a array;;
  
  external unsafe_of_array : 'a array -> 'a t = "%identity";;
  
  val get : 'a t -> component_number -> 'a;;
  
  val make : 'a -> 'a t;;
  val init : (component_number -> 'a) -> 'a t;;
  val to_list : 'a t -> 'a list;;
  val of_list : 'a list -> 'a t;;
  val iter : ('a -> unit) -> 'a t -> unit;;
  val map : ('a -> 'b) -> 'a t -> 'b t;;
  val iter_i : (component_number -> 'a -> unit) -> 'a t -> unit;;
  val map_i : (component_number -> 'a -> 'b) -> 'a t -> 'b t;;
  val iter_rails : ('a -> unit) t -> 'a t -> unit;;
  val fold_left : ('accu -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu;;
  val fold_right : ('a -> 'accu -> 'accu) -> 'a t -> 'accu -> 'accu;;
  val sort : ('a -> 'a -> int) -> 'a t -> 'a t;;
  
  val fold_left_i :
    ('accu -> component_number -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu
  ;;
  val fold_right_i :
    (component_number -> 'a -> 'accu -> 'accu) -> 'a t -> 'accu -> 'accu
  ;;
  val iter2 : ('a -> 'b -> unit) -> 'a t -> 'b t -> unit;;
  val map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t;;
  val iter_i2 : (component_number -> 'a -> 'b -> unit) -> 'a t -> 'b t -> unit;;
  val map_i2 : (component_number -> 'a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t;;
  val fold_left2 :
    ('accu -> 'a -> 'b -> 'accu) -> 'accu -> 'a t -> 'b t -> 'accu
  ;;
  val fold_right2 :
    ('a -> 'b -> 'accu -> 'accu) -> 'a t -> 'b t -> 'accu -> 'accu
  ;;
  val iter3 : ('a -> 'b -> 'c -> unit) -> 'a t -> 'b t -> 'c t -> unit;;
  val map3 : ('a -> 'b -> 'c -> 'd) -> 'a t -> 'b t -> 'c t -> 'd t;;
  val iter4 :
    ('a -> 'b -> 'c -> 'd -> unit) -> 'a t -> 'b t -> 'c t -> 'd t -> unit
  ;;
  val map4 :
    ('a -> 'b -> 'c -> 'd -> 'e) -> 'a t -> 'b t -> 'c t -> 'd t -> 'e t
  ;;
  val for_all : ('a -> bool) -> 'a t -> bool;;
  val exists : ('a -> bool) -> 'a t -> bool;;
  val find : ('a -> bool) -> 'a t -> 'a;;
  val find_all : ('a -> bool) -> 'a t -> 'a list;;
  
  val max_assoc : ('a * 'b) t -> 'a * 'b;;
  
  val print : ('a, 'a t) Format_printer.printer_lifter;;
end
;;

module type VECTOR_MAKER =
  functor (D : DIMENSION) ->
  VECTOR
;;
