#######################################################################
#                                                                     #
#                                 Pph                                 #
#                                                                     #
#                    The Portable Project Handler                     #
#                                                                     #
#               EPI Pomdapi, INRIA Paris - Rocquencourt               #
#                                                                     #
#  Copyright 2005-2014 INRIA.                                         #
#  Institut National de Recherche en Informatique et en Automatique.  #
#  All rights reserved. This file is distributed only by permission.  #
#                                                                     #
#  Francois Clement <Francois.Clement@inria.fr>                       #
#  Pierre Weis <Pierre.Weis@inria.fr>                                 #
#                                                                     #
#######################################################################

# $Id: configure_parsing.profile,v 1.26 2014/01/31 09:57:50 fclement Exp $

#
# Pph system configuration parsing.
# Used by user and devel configuration profiles ``configure.profile''.
#

CONFIGURE_OPTIONS="$@"
UNPARSED_ARGUMENTS=""

while test $# -gt 0; do
    
    case "$1" in
    
    # Installation root directory name.
    ( --install-root-dir | -install-root-dir | -ird | --prefix | -prefix )
        VAL=`expand "$2"`; shift
        INSTALL_ROOT_DIR_FROM_COMMAND_LINE="$VAL"
        ;;
    
    # Compilers.
    ( --c-compiler | -c-compiler | -cc )
        VAL=`expand "$2"`; shift
        C_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --cpp-compiler | -cpp-compiler | -cppc )
        VAL=`expand "$2"`; shift
        CPP_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --fortran-compiler | -fortran-compiler | -fc )
        VAL=`expand "$2"`; shift
        FORTRAN_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --java-compiler | -java-compiler | -jc )
        VAL=`expand "$2"`; shift
        JAVA_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --moca-compiler | -moca-compiler | -mc )
        VAL=`expand "$2"`; shift
        MOCA_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --ocaml-byte-compiler | -ocaml-byte-compiler | -obc )
        VAL=`expand "$2"`; shift
        OCAML_BYT_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --ocaml-native-compiler | -ocaml-native-compiler | -onc )
        VAL=`expand "$2"`; shift
        OCAML_BIN_COMPILER_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --ocaml-command-search-path | -ocaml-command-search-path | -ocsp )
        VAL=`expand "$2"`; shift
        OCAML_COMMAND_SEARCH_PATH_FROM_COMMAND_LINE="$VAL"
        ;;
    
    # Preprocessors.
    ( --c-preprocessor | -c-preprocessor | -cpp )
        VAL=`expand "$2"`; shift
        C_PREPROCESSOR_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --cpp-preprocessor | -cpp-preprocessor | -cpppp )
        VAL=`expand "$2"`; shift
        CPP_PREPROCESSOR_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --fortran-preprocessor | -fortran-preprocessor | -fpp )
        VAL=`expand "$2"`; shift
        FORTRAN_PREPROCESSOR_FROM_COMMAND_LINE="$VAL"
        ;;
    ( --ocaml-preprocessor | -ocaml-preprocessor | -opp )
        VAL=`expand "$2"`; shift
        OCAML_PREPROCESSOR_FROM_COMMAND_LINE="$VAL"
        ;;
    
    # For developers.
    ( --force | -force | -f )
        FORCE=true
        ;;
    
    # Operating modes.
    ( --verbosity-level | -verbosity-level | -vl )
        VAL=`expand "$2"`; shift
        VERBOSITY_LEVEL="$VAL"
        ;;
    ( --debug | -debug | -d)
        VERBOSITY_LEVEL="${DEBUG_LEVEL:?}"
        DEBUG=true
        ;;
    ( --report | -report | -r)
        VERBOSITY_LEVEL="${REPORT_LEVEL:?}"
        REPORT=true
        ;;
    ( --summary | -summary | -s)
        VERBOSITY_LEVEL="${SUMMARY_LEVEL:?}"
        SUMMARY=true
        ;;
    ( --trace | -trace | -x)
        set -x
        VERBOSITY_LEVEL="${DEBUG_LEVEL:?}"
        DEBUG=true
        ;;
    ( --see-you | -see-you | -n )
        VERBOSITY_LEVEL="${DEBUG_LEVEL:?}"
        SEE_YOU=true
        ;;
    
    # Help.
    ( --help | -help | -h )
        usage 1>&2
        exit 0
        ;;
    
    # Private tool.
    ( --private-tool | -private-tool | -pt )
        PRIVATE_TOOL_FROM_COMMAND_LINE=true
        VAL=`expand "$2"`; shift
        MIMIC_INSTALL_ROOT_DIR_FROM_COMMAND_LINE="$VAL"
        ;;
    
    ( * )
        if test -z "$UNPARSED_ARGUMENTS"; then
            UNPARSED_ARGUMENTS="$1"
        else
            UNPARSED_ARGUMENTS="$UNPARSED_ARGUMENTS $1"
        fi
        ;;

    esac
    
    shift
    
done

if test -n "$UNPARSED_ARGUMENTS"; then
    if ${PROJECT_CONFIGURATION_ADDITIONAL_OPTIONS:?}; then
        RES=`parse_additional_options $UNPARSED_ARGUMENTS`
        if test `$ECHO "$RES" | $AWK -F: '{print NF}'` != 2; then
            MSG="\
Finally, character colon (':') cannot be a safe separator
Anyway, issuing \`\`./configure -f\'\' may solve your problems..."
            internal_error "$MSG"
            exit $INTERNAL_ERROR
        fi
        UNPARSED_ARGUMENTS=`$ECHO "$RES" | $CUT -d : -f 1`
        BINDINGS_TO_EXPORT=`$ECHO "$RES" | $CUT -d : -f 2`
        eval "$BINDINGS_TO_EXPORT"
    fi
fi

if test -n "$UNPARSED_ARGUMENTS"; then
    for OPT in $UNPARSED_ARGUMENTS; do
        case "$UNPARSED_ARGUMENTS" in
        # Unknown option name.
        ( -* )
            MSG="unknown option name \"$OPT\""
            IER=${UNKNOWN_OPTION_NAME_ERROR:?}
            ;;
        # Invalid argument.
        ( * )
            MSG="invalid argument \"$OPT\""
            IER=${INVALID_ARGUMENT_ERROR:?}
            ;;
        esac
        error "$MSG"
        remedy ""
        exit $IER
    done
fi
