(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udelayed_dgraph.mli,v 1.7 2014/01/23 14:14:17 fclement Exp $ *)

(** {3 Delayed dynamic graphs} *)

(** Direct access data structure. *)

type arrow_number = Udynamic_graph.arrow_number;;

type 'a delayed_image =
  | Not_yet_computed
  | Image of 'a
;;
(** Type ['a delayed_image] is equivalent to ['a option]. *)

type ('a, 'b) delayed_arrow = ('a, 'b delayed_image) Udynamic_graph.arrow;;
(** The ``option'' indicates that the image may not yet be computed. *)

type ('a, 'b, 'c) t = private
  {
   f : 'c -> arrow_number -> 'a -> 'b;
   graph : ('a, 'b delayed_image) Udynamic_graph.t;
   to_do : arrow_number list;
  }
;;
(** First argument of function [f] of type ['c] may represent any additional
 argument parameter, other than that of type ['a]. If useless, simply put [unit]
 The todo list gathers the indices of the not-yet-computed images. *)

val make :
  ('c -> arrow_number -> 'a -> 'b) -> arrow_number -> 'a -> ('a, 'b, 'c) t
;;

val get : ('a, 'b, 'c) t -> arrow_number -> ('a, 'b) delayed_arrow;;
val set_next : ('a, 'b, 'c) t -> 'a -> ('a, 'b, 'c) t;;
val set : ('a, 'b, 'c) t -> arrow_number -> 'a -> ('a, 'b, 'c) t;;

val same : ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> bool;;

val do_it : 'c -> ('a, 'b, 'c) t -> ('a, 'b, 'c) t;;

(** {6 Converters} *)

val to_array : ('a, 'b, 'c) t -> ('a, 'b) Udynamic_graph.arrow array;;
(** Raise [Invalid_arg] when the to_do list is nonempty. *)

val of_values :
  ('c -> arrow_number -> 'a -> 'b) ->
  'a array -> ('a, 'b, 'c) t
;;
(** The resulting delayed dynamic graph is fully delayed: all images in the
 arrows are [Not_yet_computed]. *)

(** {6 Iterators} *)

val iter :
  (arrow_number -> ('a, 'b) delayed_arrow -> unit) -> ('a, 'b, 'c) t -> unit
;;
(** Raise [Invalid_arg] when the to_do list is nonempty. *)

val fold_left :
  ('accu -> arrow_number -> ('a, 'b) delayed_arrow -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;
(** Raise [Invalid_arg] when the to_do list is nonempty. *)

val iter_to_do : (arrow_number -> 'a -> unit) -> ('a, 'b, 'c) t -> unit;;
(** Iterates on the to_do list only. *)

val iter_value : (arrow_number -> 'a -> unit) -> ('a, 'b, 'c) t -> unit;;
val fold_left_value :
  ('accu -> arrow_number -> 'a -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
;;
(** Only consider the value field of the delayed arrows. *)
